<?php

  //
  // Utilities for harvesting from publicationslist.org API
  //

  // defaults:
$extradcfields = array();

if (!file_exists("config.inc.php")) {
  print "Error - config.inc.php file not found.\n";
  print "  You need to:\n";
  print "  cp config.inc.sample config.inc.php\n";
  print "  ... then edit the settings.\n";
  exit(1);

 }
require_once("config.inc.php");

$trackbackurl = $publisthome."/api/trackback.php";

// The OAI-PMH feed for local dspace:
$pmhurl     = $dspaceurl."/oai/request";

// The handle server for links to dspace items: 
$handlepfix = $dspaceurl."/xmlui/handle/";

// Full text only?  If set, don't import metadata-only records
// from publicationslist entries.
$fulltextonly = true; // false;

require_once("utfutil.php");


$cachedir = "cache";

// Get OAI cache for checking status of deposited items.
function getOaiCacheDir() {
  global $cachedir;
  $oaicache = $cachedir."/oai";
  if (!is_dir($oaicache)) { mkdirs($oaicache); }
  return $oaicache;
}

// Get submitted articles cache by name
// - items harvested but maybe not accepted yet.
function getSubmittedCacheDir() {
  global $cachedir;
  $subcache = $cachedir."/submitted";
  if (!is_dir($subcache)) { mkdirs($subcache); }
  return $subcache;
}

// Get submitted articles cache by publist ID
function getSubmittedIDCacheDir() {
  global $cachedir;
  $subcache = $cachedir."/submitted-ids";
  if (!is_dir($subcache)) { mkdirs($subcache); }
  return $subcache;
}

// submitted-ids/fr/fred/ref-1.txt --- unique filename.
function getSubmittedIDCacheFile($userid, $refid) {
  $dir = getSubmittedIDCacheDir();
  $subdir = substr($userid, 0, 2);
  return $dir."/".$subdir."/$userid/ref-$refid.txt";
}

// Get xfer cache dir for running the importer
// xfer/fred/ref-1/
function getXferCacheDir() {
  global $cachedir;
  $subcache = $cachedir."/xfer";
  if (!is_dir($subcache)) { mkdirs($subcache); }
  return $subcache;
}

// Date of last OAI fetch YYYY-MM-DD
function getOaiDateFile() {
  return getOaiCacheDir()."/_last_fetch.txt";
}

function getLastOaiDate() {
  $file = getOaiDateFile();
  if (file_exists($file)) { return file_get_contents($file); }
  return "";
}

function setLastOaiDate() {
  $date = gmdate("Y-m-d");
  $fp = fopen(getOaiDateFile(), "w");
  fwrite($fp, $date);
  fclose($fp);
}


// Date of last submission YYYY-MM-DD
function getSubmittedDateFile() {
  return getSubmittedCacheDir()."/_last_submit.txt";
}

function getLastSubmittedDate() {
  $file = getSubmittedDateFile();
  if (file_exists($file)) { return file_get_contents($file); }
  return "";
}

function setLastSubmittedDate() {
  $date = gmdate("Y-m-d");
  $fp = fopen(getSubmittedDateFile(), "w");
  fwrite($fp, $date);
  fclose($fp);
}


function chopStr($str, $max) {
  if (strlen($str) > $max) {
    $hmax = $max/2;
    $str = substr($str,0,$hmax)."___".substr($str, strlen($str)-$hmax);
  }
  return $str;
}

// Remove insig words
function sigStr($str) {
  $str = strtr($str, ",.?:-;/!\"\$%^&*(){}[]'`+", 
                     "                            ");
  $words = explode(" ", $str);

  $ret = "";
  foreach ($words as $word) {
    if (strlen($word)>1) {
      if ($ret != "") { $ret.="_"; }
      $ret .= $word;
    }
  }
  return $ret;
}

//
// Get a unique path for an article.
// 1st 3 digits of 1st surname / authors-year-title.txt
// surnames is array of surnames.
function getUniquePath($surnames, $title, $year) {

  $authors = deaccent(implode("-", $surnames));
  $authors = utf_strtolower($authors);
  $authors = strtr($authors, "'.?:", "_____");
  $authors = chopStr($authors, 80);
  $dir = substr($authors."___", 0, 3);
  $title = sigStr(deaccent($title));
  $title = chopStr($title, 80);

  // Needs a better mangle for title uniqueness...
  return $dir."/".$authors."-".$year."-".utf_strtolower($title).".txt";
}

function mkdirs($path) {
  umask(0);
  if (!file_exists($path)) {
    mkdirs(dirname($path));          
    @mkdir($path);
  }
}

function utf_strtolower($str) {
  return strtr($str, "ABCDEFGHIJKLMNOPQRSTUVWXYZ", "abcdefghijklmnopqrstuvwxyz");
}

function utf_strtoupper($str) {
  return strtr($str, "abcdefghijklmnopqrstuvwxyz", "ABCDEFGHIJKLMNOPQRSTUVWXYZ");
}

?>