<?php

  //
  // Harvesting module for fetching items from 
  // a PublicationsList.org installation into a 
  // DSpace repository.
  //
  // Written by Fred Howell, Nov 2009
  // 
  // Contact support@publicationslist.org with any questions.
  //


require_once('importutil.php');
require_once('oaiutil.php');


// Mapping of publicationslist fields to dspace dc
  $dcfields = array(
		    // "journal" => "dc.identifier.journal",
		    // "volume" => "dc.identifier.volume",
		    // "number" => "dc.identifier.issue",
		    // "doi" => "dc.identifier.doi",
		    // "pubmed" => "dc.identifier.pubmedid",
		    "howpublished" => "dc.description",
		    "address" => "dc.description",
		    "author" => "", // dc.contributor.author", 
		    "title" => "dc.title",
		    "month" => "",
		    "year" => "",
		    "booktitle" => "dc.relation.ispartof",
		    "series" => "dc.description",
		    "editor" => "", // "dc.contributor.editor",
		    "publisher" => "dc.publisher",
		    "school" => "dc.description",
		    "organization" => "dc.description",
		    "chapter" => "dc.identifier",
		    "edition" => "dc.description",
		    "pages" => "dc.description",
		    "institution"  => "dc.description",
		    "articletype" => "dc.type",
		    "isbn" => "dc.identifier.isbn",
		    "issn" => "dc.identifier.issn",

		    "refid" => "",

		    "tags" => "",//  dc.subject.classification",

		    "pdflink" => "dc.relation.uri",
		    "urllink" => "dc.relation.uri",

		    "abstract" => "dc.description.abstract",
		    "note" => "dc.description",
		    "sectionheading" => "dc.description",
		    "key" => "" // citekey

		    );



/* non-standard dc fields now set in config.inc.php
 $extradcfields = array(  
 "journal" => "dc.identifier.journal",
 // "volume" => "dc.identifier.volume",
 // "number" => "dc.identifier.issue",
 // "sectionheading" => "dc.identifier.issue",
 "doi" => "dc.identifier.doi",
 "pubmed" => "dc.identifier.pubmedid"			  
 );
*/

foreach ($extradcfields as $k=>$v) { $dcfields[$k] = $v; }

  // dc.identifier.pmcid
  // dc.citation : plain text version?
  // dc.type <> bibtex type?
  // editor, author: split
  // dc.date.issued --- month, year
  // dc.description.peerreviewed = "is {not} peer reviewed" 



function getDcValue($txt, $dcfield, $dcqual = "none") {
  return "<dcvalue element='$dcfield' qualifier='$dcqual'>".htmlspecialchars($txt, ENT_COMPAT, "UTF-8")."</dcvalue>\n";
}

// Fetch a file...
function fetchFile($url, $tgt) {
  print "fetch file $url $tgt\n";
  $curlcmd = "curl -L \"$url\" -o \"".$tgt."\"";

  print $curlcmd;
  $retary = array(); $retval = 0;
  exec($curlcmd, $retary, $retval);

}

// Import a single pub to appropriate collections
function importPub($pluser, $obj) {
  global $publisthome, $dcfields;

  $scratch = getXferCacheDir(); 
  $ary = $obj;

  $refid = $ary["refid"];
  
  $userdir = $scratch."/$pluser";
  $reftopdir = $userdir."/".$refid;
  $refdir = $reftopdir."/refdir";
  
  mkdirs($refdir);

  print "   ... importing $refid ".$ary["title"]."\n";

  // Make dublin-core.xml
  $dcfile = $refdir."/dublin_core.xml";
  $contentsfile = $refdir."/contents";

  $dctxt = "<dublin_core>\n";

  // Let's put in a relation URI to the publist ID:
  $dctxt .= getDcValue("$publisthome/$pluser#refid-".$obj["refid"], "relation", "uri");

  // Make dcdate:
  $dcdate = $obj["year"];
  $dctxt .= getDcValue($dcdate, "date", "issued"); 

  // Tags
  $tags = $obj["tags"];
  $tlist = explode(",", $tags);
  foreach ($tlist as $tag) {
    $ttag = utf_strtolower(trim($tag));
    if ($ttag) {
      $dctxt .= getDcValue($ttag, "subject", "classification"); 
    }
  }

  // Authors
  if (isset($obj["author"])) {
    $author = $obj["author"];
    $alist = splitAuthorsList($author);
    foreach ($alist as $au) {
      $austr = $au["last"].", ";
      $austr = appendSpace($austr, $au["first"]);
      $austr = appendSpace($austr, $au["middle"]);
      $dctxt .= getDcValue($austr, "contributor", "author"); 
    }
  }

  // Editors
  if (isset($obj["editor"])) {
    $editor = $obj["editor"];
    $alist = splitAuthorsList($editor);
    foreach ($alist as $au) {
      $austr = $au["last"].", ";
      $austr = appendSpace($austr, $au["first"]);
      $austr = appendSpace($austr, $au["middle"]);
      $dctxt .= getDcValue($austr, "contributor", "editor"); 
    }
  }
  

  foreach ($obj as $key=>$val) {
    if (isset($dcfields[$key])) {
      $dcfield = $dcfields[$key];
      if ($dcfield) {
	$bits = explode(".", $dcfield);
	$dctop = $bits[1];
	$dcqual = (count($bits)>2) ? $bits[2] : "none";
	if ($val != "") {
	  if ($dcfield == "dc.description") { $val = $key.": ".$val; }
	  $dctxt .= getDcValue($val, $dctop, $dcqual);
	}
      }      
    }
    else {
      print "     (ignoring dc field $key)\n";
    }

  }
  //$dctxt .= getDcValue($obj["author"], "contributor", "author"); 
  //$dctxt .= getDcValue($obj["title"], "title"); 
  //$dctxt .= getDcValue($obj["articletype"], "type"); // map from bibtex to dc... 

  $dctxt .= "</dublin_core>";
  
  $fp = fopen($dcfile, "w");
  fwrite($fp, $dctxt);
  fclose($fp);

  // Make "contents"
  $contents = "";
  if (isset($obj["attachment"]) && $obj["attachment"]) {
    $tgtname = basename($obj["attachment"]);
    $contents .= $tgtname."\n";
    print "    ... fetching $tgtname --- ".$obj["attachment"];
    fetchFile($publisthome."/data/$pluser/".$obj["attachment"], $refdir."/$tgtname");
  }

  $cfp = fopen($contentsfile, "w");
  fwrite($cfp, $contents);
  fclose($cfp);    
}

// Get unique path for publist meta item
function getUniqueObjPath($obj) {
  $title = $obj["title"];
  $year = $obj["year"];
  $author = $obj["author"];

  $alist = splitAuthorsList($author);
  $ary = array(); foreach ($alist as $au) { $ary[] = $au['last']; }

  return getUniquePath($ary, $title, $year); 
}

// Log an item as 'submitted' for duplicate detection
function logAsSubmitted($userid, $obj) {
  $spath = getUniqueObjPath($obj);
  $cdir = getSubmittedCacheDir();    
  $path = $cdir."/".$spath;
  mkdirs(dirname($path));

  if (file_exists($path)) {
    print "   $path already present";
  }
  else {
    print "   $path is new:";
    $fp = fopen($path, "w");
    $obj["userid"] = $userid; // first user id to submit it ...
    // date? repostatus?
    fwrite($fp, json_encode($obj));
    fclose($fp);
  }

  // Log it by id
  $idfile = getSubmittedIDCacheFile($userid, $obj["refid"]);
  if (file_exists($idfile)) {
    print "   $idfile already there\n";
  }
  else {
    mkdirs(dirname($idfile));
    $fp = fopen($idfile, "w");
    fwrite($fp, $spath);
    fclose($fp);
    print "   ... writing $spath to $idfile\n";
  }
}

// Check if this ID has been submitted yet by this user...
function checkIfIDSubmitted($userid, $refid) {
  $idfile = getSubmittedIDCacheFile($userid, $refid);
  // print "checking ... ".$idfile."\n";
  return file_exists($idfile);
}

// Check if submitted to repo.
function checkIfSubmitted($obj) {
  $spath = getUniqueObjPath($obj);
  $cdir = getSubmittedCacheDir();    
  $path = $cdir."/".$spath;

  if (file_exists($path)) {
    return (array) json_decode(file_get_contents($path));
  }
  else {
    return false;
  }
}

// Check if accepted in repo.
function checkIfAccepted($obj) {
  $spath = getUniqueObjPath($obj);
  $cdir = getOaiCacheDir();    
  $path = $cdir."/".$spath;

  //  print "checking acc $path ...\n";

  if (file_exists($path)) {
    return json_decode(file_get_contents($path));
  }
  else {
    return false;
  }
}

// Run the dspace importer for a single publication.
function dspaceImport($pluser, $meta) {
  global $dspacebin, $dspaceuser;
  $scratch = getXferCacheDir();

  $refid = $meta["refid"];
  $collections = $meta["repocollections"];

  $collarg = "";
  $colls = explode(";", $collections);
  foreach ($colls as $c) {  
    $c = trim($c);
    if ($c != "") {
      $collarg .= " -c $c";
    }
  }
  if ($collarg != "") {
    $cmd = $dspacebin."/dsrun org.dspace.app.itemimport.ItemImport -a -w true -e $dspaceuser $collarg -s $scratch/$pluser/$refid -m mapfile.txt";
    if (file_exists('mapfile.txt')) {
      unlink ("mapfile.txt");
    }
    print "   >>> running dspace import: $cmd\n";
    exec($cmd, $retary, $retval);
        
    print "   >>> Dspace import returned:\n";
    print implode("\n    ", $retary);

    if ($retval==0) {
      // Log to submitted ...
      print "   >>> logging as a submitted item\n";
      logAsSubmitted($pluser, $meta);
      
      trackBackSubmitted($pluser, $refid, "in review");
    }
    else {
      print "   *** Error code $retval returned - not imported ***\n";

      // track back with error code.
      trackBackSubmitted($pluser, $refid, "error during submission", false);
    }
  }
  else {
    print "   >>> not running import: no collections specified for $pluser $refid\n"; // so we'll fetch again later...
    print_r($meta);
  }
  
}

// trackback a link to given ref
function trackBackAccepted($userid, $refid, $obj) {
  global $trackbackurl, $trackbackCode;
  
  $tburl = $trackbackurl.
	    "?auth=".$trackbackCode.
	    "&userid=".$userid.
	    "&refid=".$refid.
	    "&status=accepted".
	    "&repoid=".$obj["repoid"].
	    "&repourl=".$obj["repourl"];

  print "  ... calling trackback $tburl";

  $ret = file_get_contents($tburl);
  if (substr($ret, 0,2) != "OK") {
    print $ret; // to log
  }
  else {
    print " ... OK\n";
  }
}

// Submitted by someone else...
function trackBackSubmitted($userid, $refid, $msg, $isok=true) {
  global $trackbackurl, $trackbackCode;
  
  $tburl = $trackbackurl.
	    "?auth=".$trackbackCode.
	    "&userid=".$userid.
	    "&refid=".$refid.
    "&status=".($isok ? 'harvested' : 'error').
    "&repomsg=".rawurlencode($msg);

  print "Calling trackback $tburl ...";

  $ret = file_get_contents($tburl);
  if (substr($ret, 0,2) != "OK") {
    print $ret; // to log
  }
  else {
    print "Sent trackback OK\n";
  }
}

// Check for updated refs from a user.
function getUpdatedRefs($userid, $datetime, $doall = false) {
  global $publisthome, $fulltextonly;
  print "\n  Getting updated publications list entries for: $userid  ($datetime)\n";

  $url = $publisthome."/api/getUpdatedRefs.php?userid=$userid";
  $lastdate = getLastSubmittedDate();
  if ($lastdate && !$doall) {
    $url .= "&from=$lastdate";
    print "  (since $lastdate)\n";
  }

  $txt = file_get_contents($url);
  $obj = json_decode($txt);  

  // print "Got $url $txt\n";
  // refid, status, datetime, {meta}
  foreach ($obj as $item) {
    $ary = (array)$item;
    $refid = $ary["refid"];
    if (isset($ary["meta"])) {
      $meta = (array)$ary["meta"];

      // check repostatus first ...
      if (isset($meta["repostatus"]) && $meta["repostatus"]=="accepted" || $meta["repostatus"]=="error") {
	print "  ... nothing to do for $userid $refid as repostatus = ".$meta["repostatus"]."\n";
      }
      else {
	// Has it recently been accepted?
	$isaccepted = checkIfAccepted($meta);
	if ($isaccepted) {
	  if (isset($meta["repocollections"]) && $meta["repocollections"]!= "-") {
	    print "  ... $userid $refid now accepted in repository! Sending trackback\n";
	    // ping back to the user that this item is there!
	    
	    trackBackAccepted($userid, $refid, (array)$isaccepted);	    
	  }
	  else {
	    print "  ... $userid $refid accepted in repository but they don't want a trackback link\n";
	  }
	}
	else {
	  // Has this ID already been checked?
	  if (checkIfIDSubmitted($userid, $refid)) {
	    print "  ... $userid $refid already submitted ... waiting ...\n";
	  }
	  else {
	    // submitted by someone else?
	    $issubmitted = checkIfSubmitted($meta);
	    if ($issubmitted) {
	      print "  ... $userid $refid already submitted to repository by someone else.\n";

	      logAsSubmitted($userid, $meta);
	      trackBackSubmitted($userid, $refid, "Submitted by ".$issubmitted["userid"]);	
	    }
	    else {	      
	      if (isset($meta["repocollections"]) && $meta["repocollections"]) {		
		if ((isset($meta['attachment']) && $meta['attachment']) || !$fulltextonly) {
		  print "  >>> Submitting a new entry: $userid $refid...\n";

		  importPub($userid, $meta);
		  dspaceImport($userid, $meta);
		}
		else {
		  print "  ... not importing $userid $refid as no full text attached\n";

		}
	      }
	      else {
		print "  ... not importing $userid $refid as no collection chosen\n";
	      }
	    }
	  }
	}
      }
    }
  }
}


// Check for updated lists.
function getUpdatedLists($doall = false) {
  global $publisthome;

  $url = $publisthome."/api/getUpdatedLists.php";

  $lastdate = getLastSubmittedDate();
  if ($lastdate && !$doall) {
    $url .= "?from=$lastdate";
  }

  $txt = file_get_contents($url);

  $obj = json_decode($txt);

  // map of user to date
  foreach ($obj as $userid => $datetime) {
    getUpdatedRefs($userid, $datetime, $doall);
  }  

  // Store last submitted date..
  setLastSubmittedDate();
}

$lockfile = '_import_lock.tmp';

if (file_exists($lockfile)) {
  $mtime = filemtime($lockfile);
  $now = time();
  if ($now - $mtime < 60 * 10) {
    print "The $lockfile file indicates that another import process\nis already running\n - you can remove it and try again if this isn't so\n";
    exit(1);
  }
 }
touch($lockfile);

print "1. Checking for new items made live in the repository:\n";
doOaiScan(false);

print "\n2. Getting updated publications lists:\n";
getUpdatedLists(false);

print "\n3. Checking for any new items made live immediately:\n";
doOaiScan(false);

unlink($lockfile);

?>