<?php
  // bits of publist ...

require_once("harvestutil.php");


// Append a string with space sep
function appendSpace($old, $new) {
  if ($new=="") { return $old; }
  if ($old != "") { 
	return "$old $new"; 
  }
  else {
	return $new;
  }
}

function validSurname($str) {
  if (strlen($str) <=1) { return false; }

  $init = getInitialChar($str);

  // Initial char must be upper case
  // Length > 1
  if ($init == utf_strtoupper($init)) {
	if (strlen($str) > 3) { 	  
	  return true; 
	}

	if ($str != utf_strtoupper($str)) {
	  return true;
	}
  }
  
  return false;
}

// true for "" A AB ABC le de van
function validInitials($str) {
  if ($str == "") { return true; }
  $init = getInitialChar($str);
  $sameupper =($str == utf_strtoupper($str));
  $samelower = ($str == utf_strtolower($str));
  if ($sameupper || $samelower) {
	if ((strlen($str) / strlen($init)) < 4) {
	  return true;
	}
  }
  return false;
}

// Parse an author and return an array
// Passed previous author in case only initials which need to get prepended to prev one.
function newSplitAuthor($prevauthor, $author) {
  $author = trim(strtr($author, ".", " ")); // Get rid of "."s
  $words = explode(" ", $author);

  // Remove null entries and (brackets )
  $words2 = array();
  $ignore = 0;
  foreach ($words as $word) {
	$w = trim($word);
   
	if ($w != "") { 
	  // Remove entries in ( )
	  if (strpos($w, "(") !== false) {
		$ignore ++;
	  }
	  else if ((strpos($w, ")") !== false)) {
		$ignore --;
	  }
	  else if ($ignore <= 0) {
		$words2[] = $w;
	  }
	}
  }
  $words = $words2;

  $num = count($words);

  // Empty: return false
  if ($num == 0) { return false; }

  $first = "";
  $middle = "";
  $last = "";

  // --- Just 1 word: either initials for previous entry, or a surname.
  $last = $words[$num -1];
  $lastvalid = validSurname($last);
  $lastvalidinitial = validInitials($last);

  $allinitials = true;
  for ($i=0; $i<$num; $i++) {
	if (!validInitials($words[$i])) {
	  $allinitials = false;
	}
  }

  if ($num == 1) {
	$initials = validInitials($last);
	if ($lastvalid || $prevauthor == false || !$initials) {
	  return array("first"=>"", "middle"=>"", "last"=>$last);
	}	
	else {
	  $prevauthor["first"] = prependSpace($prevauthor["first"], $last); 
	  $prevauthor["extended"] = true;
	  return $prevauthor;
	}	
  }
  else {
	// Is 1st valid?
	$first = $words[0];
	$firstvalid = validSurname($first);
	
	if ($firstvalid && !$lastvalid) {
	  // we're looking at Howell F W - do a swap.
	  // De Castro S  --> S De Castro 
	  $sncount = 1;
	  for ($n=1; $n<$num; $n++) {
		if (validSurname($words[$n])) {
		  $sncount = $n + 1;
		}
		else {
		  break;
		}
	  }
	  
	  $sns = array_splice($words, 0, $sncount);
	  foreach ($sns as $sn) {
		$words[] = $sn;
	  }

	  $first = $words[0];
	  $last = $words[$num-1];
	}
	else if ($allinitials && $prevauthor != false) {
	  $extra = $words[0];

	  for ($i=1; $i<$num; $i++) {
		$extra = appendSpace($extra, $words[$i]);
	  }	

	  $prevauthor["first"] = prependSpace($prevauthor["first"], $extra);	  
	  $prevauthor["extended"] = true;
	  return $prevauthor;
	}   
  }
  

  // OK, should now be regular....
  $first = "";
  $middle = "";
  $last = "";

  if ($num > 0) { $last = $words[$num -1]; }
  if ($num > 1) { $first = $words[0]; }
  if ($num > 2) { 
	// Check for van, de, le
    $mid = $words[$num - 2];
    $max = $num - 2;
    if ($mid == utf_strtolower($mid)) {
      $last = $mid." ".$last;
      $max --;
    }
    if ($num > 3) {
      $mid = $words[$num - 3];
      if ($mid == utf_strtolower($mid)) {
        $last = $mid." ".$last;
        $max --;
      }      
    }

    for ($i=1; $i<=$max; $i++) {
      if ($middle != "" && $words[$i] != "") { $middle .= " "; }
      $middle .= $words[$i];
    }
  }  

  // Check for first name all caps and no middle
  if (validInitials($first) && $middle == "" && $first == utf_strtoupper($first)) {
	// explode in a UTF friendly way?
	$rest = $first;
	$init = getInitialChar($rest);
	$first = $init;

	do {
	  $len = strlen($init);
	  $rest = substr($rest, $len); 
	  if ($rest != "") {
		$init = getInitialChar($rest);
		if ($init) {
		  $middle = appendSpace($middle, $init);
		}
	  }
	} while ($rest != "" && $len > 0);
  }
  
  return array("first"=>trim($first), "middle"=>trim($middle), "last"=>trim($last));
}

function getInitialChar($str) {
  if ($str == "") { return ""; }
  $c1 = substr($str, 0, 1);  
  $chr = ord($c1);
  $l = 1;
  if ($chr & 0x80) {
    $chr <<=1;
    while ($chr & 0x80) {
      $l++;
      $chr <<= 1;
    }
  }
  return substr($str, 0, $l);  
}

function splitAuthorsList($str) {
  // Get rid of '&'
  $str = strtr($str, "&;:", ",, ");

  // Get rid of ' and '
  $str = str_replace( " and ", ",", $str);

  // Split on commas
  $bits = explode( ",", $str);

  $ret = array();

  // OK, now we have the authors.   
  $prevauthor = false;
  foreach ($bits as $bit) {
	$bit = trim($bit);
	if ($bit) {
	  $nextauthor = newSplitAuthor($prevauthor, $bit);
	  if (isset($nextauthor["extended"])) {
		$ret[count($ret) -1] = $nextauthor; // replace with extended version
		$prevauthor = false;
	  }
	  else {
		$ret[] = $nextauthor;
		$prevauthor = $nextauthor;
	  }
	}
  }

  return $ret;
}


?>

