<?php

require_once("harvestutil.php");


function getSurnames($mod) {
  $ret = array();
  foreach ($mod->mods_name as $obj) {
    //  if ($obj->mods_role->mods_roleTerm == "author") {
      $namepart = (string)$obj->mods_namePart;
      $cpos = strpos($namepart, ",");
      $surname = substr($namepart, 0, $cpos);
      $ret[] = utf_strtolower($surname);
      //}
  }
  return $ret;
}

function getIssuedDate($mod) {
  foreach ($mod->mods_originInfo as $oinfo) {
    if ($oinfo->mods_dateIssued) {
      return (string) $oinfo->mods_dateIssued;
    }
  }
}

function getPublistUrl($mod) {
  global $publisthome;
  foreach ($mod->mods_relatedItem as $rel) {
    $u = "".$rel->mods_location->mods_url;
    
    if ($u && strpos($u, $publisthome)!==false) {
      return $u;
    }
    
  }
  return "";
}

// Get the user and refid from a publist url
// http://publicationslist.org/fred#ref-123
function parsePublistUrl($url) {
  $bits = explode("#", $url);
  $userid = substr($bits[0], strrpos($bits[0], "/")+1);
  $refid = substr($bits[1], strrpos($bits[1], "-")+1);
  $ret = array("userid"=>$userid, "refid"=>$refid);
  return $ret;
}

//
// Check local repo and build index of duplicates.
// 
function doOaiScan($doall=false) {
  global $pmhurl, $handlepfix, $trackbackurl, $trackbackCode;
  
  print "Doing OAI scan for new items:\n";
  // Check since ...
  $lastdate = getLastOaiDate();
  $resumption = "";

  do {
    $url = $pmhurl."?verb=ListRecords&metadataPrefix=mets";
    if (!$doall && $lastdate) {
      $url .= "&from=$lastdate";
      print "  fetching new items added since $lastdate\n";
    }
    
    if ($resumption) {
      $url = $pmhurl."?verb=ListRecords&resumptionToken=".rawurlencode($resumption);
      print "Resuming using $resumption\n";
    }
   
    print "$url\n";
    $txt = file_get_contents($url);

    $txt = str_replace("<mods:", "<mods_", $txt);
    $txt = str_replace("</mods:", "</mods_", $txt);
   
    try {
      $xml = new SimpleXMLElement($txt);	
    } catch (Exception $e) {
      print "Problem parsing $txt as xml";
      return false;
    }
    
    $lr = $xml->ListRecords;
    
    if (!$lr->record) {
      // No new records?
      if ($xml->error && $xml->error["code"] == 'noRecordsMatch') {
	print "No new records found.";
      }
      else {
	print "Error $txt\n --- ".$lr->record;
      }
    }
    else {
     foreach ($lr->record as $rec) {
      $hdr = $rec->header;
      $meta = $rec->metadata;
      
      $oid = $hdr->identifier;  
      $oaidate = "".$hdr->datestamp;
      $handle = substr($oid, strrpos($oid, ":")+1);
      
      $sets = array();
      foreach ($hdr->setSpec as $spec) {
	$sets[] = "".$spec;
      }
      
      $mods = $meta->mets->dmdSec->mdWrap->xmlData;
      
      $surnames = getSurnames($mods);
      $title = $mods->mods_titleInfo;
      $title = utf_strtolower($title);
      //      $title = strtr($title, ".,", "  ");
      //$title = str_replace(" ", "_", $title); // just 1st words??
      
      $date = getIssuedDate($mods);
      $year = substr($date, 0, 4);
      $genre= "".$mods->mods_genre;
    
      $publistid = getPublistUrl($mods);

      
      // Get the cached file ...
      $dcheck = getUniquePath($surnames, $title, $year); 
      $cdir = getOaiCacheDir();
      
      $path = $cdir."/".$dcheck;
      mkdirs(dirname($path));


      if (file_exists($path)) {
	print ".";
      }
      else {
	print "$path is new:\n";

	// Update it.
	$jobj = array( "publistid"=>$publistid,
		       "repoid" => $handle,
		       "reposets" => implode(",",$sets),
		       "repourl" => $handlepfix.$handle,
		       "oaidate" => $oaidate);

	$fp = fopen($path, "w");
	fwrite($fp, json_encode($jobj));
	fclose($fp);
	
	if ($publistid) {
	  $obj = parsePublistUrl($publistid);

	  $tburl = $trackbackurl.
	    "?auth=".$trackbackCode.
	    "&userid=".$obj['userid'].
	    "&refid=".$obj['refid'].
	    "&status=accepted".
	    "&repoid=".$handle.
	    "&repourl=".$jobj["repourl"];

	  print "Calling trackback $tburl ... ";

	  $ret = file_get_contents($tburl);
	  if (substr($ret, 0,2) != "OK") {
	    print "\n".$ret."\n"; // to log
	  }
	  else {
	    print "OK\n";
	  }
	}
	
      }
	
      /*
      if ($publistid) { 
	print " ===> Sourced from publist: $publistid\n";
      }
      print " ===> Repo handle: $handle\n";
      print " ===> Repo sets: ".implode(",",$sets)."\n";
      print " ===> Repo url: ".$handlepfix.$handle."\n";
      print " ===> OAI date: ".$oaidate."\n";
      */
     } // foreach
    }
    if (isset($lr->resumptionToken)) {
      $resumption = "".$lr->resumptionToken;
    }
    else {
      $resumption = "";
    }

  } while ($resumption);

  setLastOaiDate();
  print "\n";
}


?>
